MODULE DateiBoxRL EXPORTS Main;

(* Dieses Programm dient der Verwaltung einer Boxrangliste.
   Autor           : Moritz Schnizler, Thomas von der Maen, RWTH Aachen
   Umgebung        : PM-3, Windows 2000
   Erstellt        : 29.11.00  Letzte Aenderung: 30.11.00
*)


IMPORT SIO;   (* Importiere Operationen fuer die Ein-/Ausgabe *)
IMPORT Text;  (* Importiere Operationen fuer Texte *)

IMPORT IO;     (* Importiere notwendige Operationen um Dateien zu oeffnen*)
IMPORT Rd, Wr; (* Importiere Operationen um Dateien zu lesen bzw. zu schreiben *)


TYPE Name = RECORD
              vorname : TEXT;
              nachname: TEXT;
            END;

     Boxer = RECORD
                 name       : Name;
                 nation     : TEXT;
                 gewicht    : CARDINAL;
               END;

     RanglisteRef = REF RanglistenElement;
     
     RanglistenElement = RECORD
                           boxer    : Boxer;
                           naechster: RanglisteRef;
                         END;


VAR rangliste    : RanglisteRef; (* Anker der Club-Rangliste *)
    boxer,
    gewinner,
    verlierer    : Boxer;
    auswahl      : CHAR;
    dummy        : TEXT;


PROCEDURE InitialisiereRL(VAR liste: RanglisteRef)=
(* Initialisiert die Datenstruktur fuer eine Rangliste *)
BEGIN
  liste := NIL; (* Setze Anker der Rangliste auf NIL *)
END InitialisiereRL;


PROCEDURE SucheBoxerRL(liste: RanglisteRef; boxer: Boxer): RanglisteRef=
(* Sucht den gegebenen Boxer in der Rangliste liste und gibt einen Zeiger
   auf ihn zurueck. Der Zeiger hat den Wert NIL, wenn der Boxer nicht in der 
   Liste vorhanden ist. *)

VAR resRLRef: RanglisteRef;
    gefunden: BOOLEAN;

BEGIN
   gefunden := FALSE;
   resRLRef := liste;
   WHILE (resRLRef # NIL) AND NOT gefunden DO

     (* Pruefe, ob der referenzierte Boxer der gesuchte ist *)
     IF (Text.Equal(resRLRef^.boxer.name.vorname, boxer.name.vorname)) AND
        (Text.Equal(resRLRef^.boxer.name.nachname, boxer.name.nachname)) AND
        (Text.Equal(resRLRef^.boxer.nation,  boxer.nation)) AND
        (resRLRef^.boxer.gewicht = boxer.gewicht)
     THEN
       gefunden := TRUE;
     ELSE
       resRLRef := resRLRef^.naechster;
     END;

   END; (* WHILE *)

   RETURN resRLRef;
END SucheBoxerRL;


PROCEDURE SucheVorgaengerRL(liste: RanglisteRef; boxer: Boxer): RanglisteRef=
(* Sucht den Vorgaenger des gegebenen Boxers in der Rangliste liste und gibt
   einen Zeiger auf ihn zurueck. Der Zeiger hat den Wert NIL, wenn der Boxer keinen
   Vorgaenger hat, d.h. wenn er das erste Element oder nicht in der Liste ist. *)

VAR vorRLRef, aktRLRef: RanglisteRef;
    gefunden          : BOOLEAN;

BEGIN
   gefunden := FALSE;
   vorRLRef := NIL;    (* Es gibt keinen Vorgaenger des ersten Elements! *)
   aktRLRef := liste;
   WHILE (aktRLRef # NIL) AND NOT gefunden DO
     
     (* Pruefe, ob der referenzierte Boxer der gesuchte ist *)
     IF (Text.Equal(aktRLRef^.boxer.name.vorname, boxer.name.vorname)) AND
        (Text.Equal(aktRLRef^.boxer.name.nachname, boxer.name.nachname)) AND
        (Text.Equal(aktRLRef^.boxer.nation, boxer.nation)) AND
        (aktRLRef^.boxer.gewicht = boxer.gewicht)
     THEN
       gefunden := TRUE;
     ELSE
       vorRLRef := aktRLRef;
       aktRLRef := aktRLRef^.naechster; 
     END;

   END; (* WHILE *)

   (* Zeiger auf den Vorgaenger ist NIL, wenn Boxer nicht in der Liste ist *)
   IF NOT gefunden THEN vorRLRef := NIL; END;

   RETURN vorRLRef;
END SucheVorgaengerRL;


PROCEDURE DruckeRL(liste: RanglisteRef)=
(* Gibt die als Parameter gegebene Rangliste in absteigender Rangfolge aus *)

VAR ranglisteRef: RanglisteRef;
    rang        : CARDINAL;

BEGIN
  (* Gib Kopfkommentar der Rangliste aus *)
  SIO.PutLine(" *** Box-Rangliste *** "); 
  SIO.Nl();
 
  ranglisteRef := liste;
  rang := 1;
  WHILE (ranglisteRef # NIL) DO

    (* Gib Rang und Daten des Boxers aus *)
    SIO.PutInt(rang); SIO.PutText(". ");
    SIO.PutText(ranglisteRef^.boxer.name.nachname & ", ");
    SIO.PutText(ranglisteRef^.boxer.name.vorname & "   ");
    SIO.PutText("Nation: ");
    SIO.PutText(ranglisteRef^.boxer.nation & "  ");
    SIO.PutText("Gewicht: ");
    SIO.PutInt(ranglisteRef^.boxer.gewicht);
    SIO.Nl();

    (* Inkrementiere Boxer und Rang *)
    ranglisteRef := ranglisteRef^.naechster;
    rang := rang + 1;

  END; (* WHILE *)
END DruckeRL;


PROCEDURE SchreibeDateiRL(dateiName: TEXT; liste: RanglisteRef)=
(* Gibt die als Parameter gegebene Rangliste in absteigender Rangfolge in
   der Datei mit dem uebergebenen Namen aus *)

VAR ranglisteRef: RanglisteRef;
    rang        : CARDINAL;

    datei: Wr.T;

BEGIN
  datei := IO.OpenWrite(dateiName);

  (* Gib Kopfkommentar der Rangliste aus *)
  SIO.PutLine(" *** Box-Rangliste *** ", datei); 
  SIO.Nl(datei);
 
  ranglisteRef := liste; rang := 1;
  WHILE (ranglisteRef # NIL) DO

    (* Gib Rang und Daten des Boxers aus *)
    SIO.PutInt(rang, 10, datei); SIO.PutText(". ", datei);
    SIO.PutText(ranglisteRef^.boxer.name.nachname & ", ", datei);
    SIO.PutText(ranglisteRef^.boxer.name.vorname & "   ", datei);
    SIO.PutText("Nation: ", datei);
    SIO.PutText(ranglisteRef^.boxer.nation & "  ", datei);
    SIO.PutText("Gewicht: ", datei);
    SIO.PutInt(ranglisteRef^.boxer.gewicht, 10, datei);
    SIO.Nl(datei);

    (* Inkrementiere Boxer und Rang *)
    ranglisteRef := ranglisteRef^.naechster; rang := rang + 1;

  END; (* WHILE *)

  Wr.Close(datei);
  
END SchreibeDateiRL;


PROCEDURE LoescheBoxerRL(VAR liste: RanglisteRef; boxer: Boxer)=
(* Loescht den gegebenen Boxer aus der Rangliste liste, falls vorhanden. *)

VAR ranglisteRef, 
    vorgaengerRef: RanglisteRef;

BEGIN
  (* Suche den Boxer in der Liste *)
  ranglisteRef := SucheBoxerRL(liste, boxer);

  IF (ranglisteRef # NIL) THEN
    
    (* Vorhandenen Boxer aus der Rangliste loeschen *)
    vorgaengerRef := SucheVorgaengerRL(liste, boxer);
    IF vorgaengerRef = NIL THEN
      (* Erstes Element in Rangliste, daher kein Vorgaenger *)
      liste := ranglisteRef^.naechster;
    ELSE
      vorgaengerRef^.naechster := ranglisteRef^.naechster;
    END;

  ELSE
    SIO.PutLine("FEHLER: Boxer kommt nicht in der Liste vor!");
  END;
END LoescheBoxerRL;
      

PROCEDURE EinfuegenBoxerRL(VAR liste: RanglisteRef; boxer: Boxer)=
(* Fuegt den gegebenen Boxer am Ende der Rangliste liste ein *)

VAR ranglisteRef, aktRLRef: RanglisteRef;

BEGIN
  (* Pruefe, ob Boxer nicht schon in Rangliste enthalten *)
  ranglisteRef := SucheBoxerRL(liste, boxer);
 
  IF (ranglisteRef = NIL) THEN

    (* Erzeuge neues Ranglisten-Element fuer den Boxer *)
    ranglisteRef := NEW(RanglisteRef);
    ranglisteRef^.boxer := boxer;
    ranglisteRef^.naechster := NIL;

    (* Fuege Element an Ende der Liste an *)  
    IF (liste = NIL) THEN
      (* Liste ist noch leer *)
      liste := ranglisteRef;
    ELSE
      (* Suche Ende der Liste *)
      aktRLRef := liste;
      WHILE (aktRLRef^.naechster # NIL) DO
        aktRLRef := aktRLRef^.naechster;
      END;
      (* Haenge Boxer ans Ende an *)
      aktRLRef^.naechster := ranglisteRef;
    END;
  ELSE
    SIO.PutLine("FEHLER: Boxer ist bereits in der Rangliste!");
  END;
END EinfuegenBoxerRL;


PROCEDURE AKommtHinterBInRL(aRef, bRef: RanglisteRef): BOOLEAN=
(* Prueft, ob das durch Zeiger aRef referenzierte Element hinter dem durch bRef 
   referenzierten in der Rangliste vorkommt *)

VAR ranglisteRef: RanglisteRef;
  
BEGIN
  ranglisteRef := aRef;
  WHILE (ranglisteRef # NIL) AND (ranglisteRef # bRef) DO
    ranglisteRef := ranglisteRef^.naechster;
  END;

  RETURN NOT (ranglisteRef = bRef);
END AKommtHinterBInRL;


PROCEDURE AktualisiereRL(VAR liste: RanglisteRef; gewinner, verlierer: Boxer)=
(* Aendert die Rangfolge in der Rangliste entsprechend dem gegebenen 
   Boxergebnis (Gewinner, Verlierer) *)

VAR gewinnerRef, verliererRef      : RanglisteRef;
    vorVerliererRef: RanglisteRef;

BEGIN
  (* Suche zunaechst Gewinner und Verlierer in der Rangliste *)
  gewinnerRef := SucheBoxerRL(liste, gewinner);
  verliererRef := SucheBoxerRL(liste, verlierer);

  IF (gewinnerRef = NIL) OR (verliererRef = NIL) THEN
    SIO.PutLine("FEHLER: Wenigstens einer der Boxer ist nicht in der Rangliste!");
  ELSE
    
    (* Pruefe, ob nicht Gewinner ohnehin vor Verlierer in der Rangliste! *)
    IF NOT AKommtHinterBInRL(gewinnerRef, verliererRef) THEN
      SIO.PutLine("Keine Veraenderung in der Rangliste!");
    ELSE

      (* Fuege Verlierer hinter Gewinner in Rangliste ein *)

      (* Suche die jeweiligen Vorgaenger in der Rangliste *)
      vorVerliererRef := SucheVorgaengerRL(liste, verlierer);

      (* Entferne Verlierer aus ursprnglicher Position *)
      IF ( vorVerliererRef = NIL) THEN
        (* Kein Vorgaenger, da ganz am Anfang der Liste *)
        liste := verliererRef^.naechster;
      ELSE
        vorVerliererRef^.naechster := verliererRef^.naechster;
      END;

      (* Hnge Verlierer hinter Gewinner ein *)
      verliererRef^.naechster := gewinnerRef^.naechster;
      gewinnerRef^.naechster := verliererRef;     
    END;
  END;
END AktualisiereRL;


PROCEDURE LeseBoxerEin(nachricht: TEXT): Boxer=
(* Fragt den Benutzer nach den Daten eines Boxers und gibt
   anschliessend einen entsprechenden Boxer zurueck *)

VAR resBoxer     : Boxer;
    dummy          : TEXT;

BEGIN
  SIO.PutLine(nachricht);
  SIO.PutText("Nachname   : "); 
  resBoxer.name.nachname := SIO.GetLine();
  SIO.PutText("Vorname    : "); 
  resBoxer.name.vorname := SIO.GetLine();
  SIO.PutText("Nation     : ");
  resBoxer.nation := SIO.GetLine();
  SIO.PutText("Gewicht    : ");
  resBoxer.gewicht := SIO.GetInt();
  
  (* Lies RETURN nach Gewicht aus der Eingabe! *)
  dummy := SIO.GetLine();                  
  
  RETURN resBoxer;
END LeseBoxerEin;

PROCEDURE DruckeMenue()=
(* Hilfsprozedur, die das Befehlsmenue ausgibt *)
BEGIN
  SIO.Nl();
  SIO.PutLine("Z : Zeige Rangliste an");
  SIO.PutLine("S : Schreibe Rangliste in Datei");
  SIO.PutLine("F : Fuege einen Boxer ein");
  SIO.PutLine("L : Loesche einen Boxer");
  SIO.PutLine("G : Kampf gemacht");
  SIO.PutLine("E : Exit");
  SIO.Nl();
END DruckeMenue;


BEGIN
  InitialisiereRL(rangliste);
  REPEAT
    DruckeMenue();

    SIO.PutText("Auswahl: ");
    auswahl := SIO.GetChar();
    dummy := SIO.GetLine(); (* Lies RETURN aus dem Eingabepuffer! *)
    SIO.Nl();

    CASE auswahl OF
    |  'Z', 'z' => DruckeRL(rangliste);
    |  'S', 's' => SchreibeDateiRL("Rangliste", rangliste);  (* Neuer Befehl, um Datei zu schreiben *)
    |  'F', 'f' => boxer := LeseBoxerEin("Boxer Einfuegen"); SIO.Nl();
                   EinfuegenBoxerRL(rangliste, boxer);
    |  'L', 'l' => boxer := LeseBoxerEin("Boxer Loeschen"); SIO.Nl();
                   LoescheBoxerRL(rangliste, boxer);
    |  'G', 'g' => gewinner := LeseBoxerEin("Sieger des Spiels"); SIO.Nl();
                   verlierer := LeseBoxerEin("Verlierer des Spiels"); SIO.Nl();
                   AktualisiereRL(rangliste, gewinner, verlierer);
    |  'E', 'e' => (* Nichts machen, gleich ist ohnehin alles zu Ende! *)
    ELSE
      SIO.PutLine("Ungueltiger Befehl!");
    END (* CASE *)

  UNTIL (auswahl = 'e') OR (auswahl = 'E');

END DateiBoxRL.

