INTERFACE Geldbetrag;

(* Schnittstelle eines ADTs fuer Geldbetraege.
   Autor           : Antje Nowack
   Umgebung  : PM3, Windows 95
   Erstellt        : 04. 01. 2001  Letzte Aenderung: 09.01. 2001 *)

IMPORT Waehrung; (* Importiere ADT Waehrung *)

EXCEPTION NegativeResult;
                        DifferentCurrencies;
                        InvalidCurrency;
                        DivisionByZero;
                        NegativeInput;
                        FirstArgNoValue;
                        SecondArgNoValue;
                       ArgNoValue;

TYPE T <: REFANY;  (* Abstrakter Datentyp Waehrung *)

PROCEDURE Init(VAR betrag: T);
(* Initialisierung *)

PROCEDURE Tausche(betrag : T; waehrung: Waehrung.T): T RAISES{InvalidCurrency, ArgNoValue};
(* Tauscht den gegebenen Geldbetrag in die angegebene Waehrung und gibt den neuen Geldbetrag zurueck. *)

PROCEDURE Erzeuge(skalar: REAL; waehrung: Waehrung.T): T RAISES{InvalidCurrency, NegativeInput};
(* Erzeugt einen neuen Geldbetrag mit dem gegebenen Wert in der angegebenen Waehrung. *)

PROCEDURE Addiere(aBetrag, bBetrag: T): T RAISES{DifferentCurrencies, InvalidCurrency, FirstArgNoValue, SecondArgNoValue};
(* Addiert den Geldetrag bBetrag zum Betrag aBetrag und gibt das
   Ergebnis in der gemeinsamen Waehrung zurueck. *)

PROCEDURE Subtrahiere(aBetrag, bBetrag: T): T RAISES{NegativeResult, DifferentCurrencies, InvalidCurrency, FirstArgNoValue, SecondArgNoValue}; 
(* Subtrahiert den Geldbetrag bBetrag vom Betrag aBetrag und gibt
   das Ergebnis in der gemeinsamen Waehrung zurueck. Das Ergebnis darf nicht negativ sein.*)

PROCEDURE Multipliziere(betrag: T; skalar: REAL): T  RAISES{NegativeResult, InvalidCurrency, ArgNoValue};
(* Multipliziert den Geldbetrag betrag mit dem gegebenen Skalarwert und gibt das Ergebnis in der Waehrung 
    von betrag zurueck. Das Skalar darf nicht negativ sein. *)

PROCEDURE Dividiere(betrag: T; skalar: REAL): T  RAISES{NegativeResult, InvalidCurrency, DivisionByZero, ArgNoValue};
(* Dividiert den Geldbetrag betrag durch den gegebenen Skalarwert  und gibt das Ergebnis in der Waehrung von 
     betrag zurueck. Das Skalar darf nicht negativ sein und darf nicht gleich Null sein. *)

PROCEDURE Print(betrag: T) RAISES{InvalidCurrency, ArgNoValue};
(* Gibt den Wert und die Waehrungseinheit aus. *)

PROCEDURE Gleich(aBetrag, bBetrag: T): BOOLEAN RAISES{DifferentCurrencies, InvalidCurrency, FirstArgNoValue, SecondArgNoValue};
(* Prueft, ob die beiden Geldbetraege gleich gross sind. *)

PROCEDURE Kleiner(aBetrag, bBetrag: T): BOOLEAN RAISES{DifferentCurrencies, InvalidCurrency, FirstArgNoValue, SecondArgNoValue};
(* Prueft, ob der Geldbetrag aBetrag kleiner als bBetrag ist. *)

PROCEDURE Groesser(aBetrag, bBetrag: T): BOOLEAN RAISES{DifferentCurrencies, InvalidCurrency, FirstArgNoValue, SecondArgNoValue};
(* Prueft, ob der Geldbetrag aBetrag groesser als bBetrag ist. *)

END Geldbetrag.
